package com.tanpu.community.controller;

import com.tanpu.community.api.beans.TopicBriefInfoDTO;
import com.tanpu.community.api.beans.TopicDTO;
import com.tanpu.community.api.beans.TopicDataAnalysDTO;
import com.tanpu.community.manager.TopicManager;
import com.tanpu.community.model.req.topic.TopicConcealReq;
import com.tanpu.community.model.req.topic.TopicModifyMountReq;
import com.tanpu.community.model.req.topic.TopicTopReq;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.MissingServletRequestParameterException;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@RestController
@Slf4j
@RequestMapping(value = "/api/topic")
public class TopicController {

    @Autowired
    private TopicManager topicManager;


    @GetMapping(value = "/queryTopicInfoList")
    @ApiOperation("APP全部话题页面，可搜索")
    @ResponseBody
    public List<TopicDTO> getTopicBriefInfoList(){
        List<TopicDTO> allTopic = topicManager.getAllTopicDetail();
        return allTopic;
    }

    @GetMapping(value = "/admin/queryTopicDetailList")
    @ApiOperation("查询所有的话题详情")
    @ResponseBody
    public List<TopicBriefInfoDTO> getAllTopicList(){
        List<TopicBriefInfoDTO> allTopic = topicManager.getAllTopicBriefInfo();
        return allTopic;
    }


    @PostMapping(value="/admin/insertTopic")
    @ApiOperation("新增话题")
    @ResponseBody
    public String addTopic(@RequestParam String topicTitle){
        //TODO:获取登录用户信息
        String userId = "liujm";
        topicManager.insertTopic(topicTitle,userId);
        return "success";
    }

    @ApiOperation("单个话题数据总览")
    @PostMapping("/admin/selectOne")
    @ResponseBody
    public String selectOne(@RequestParam String topicId) throws MissingServletRequestParameterException {
        if (StringUtils.isEmpty(topicId)){
            throw new MissingServletRequestParameterException("topicId","String");
        }
        TopicDTO topicDTO=topicManager.getDetail(topicId);
        return "success";
    }

    @PostMapping(value = "/admin/setTop")
    @ApiOperation("顶置/取消顶置话题")
    @ResponseBody
    public String setTopTopic(@RequestBody TopicTopReq req) throws MissingServletRequestParameterException {
        topicManager.setTopTopic(req.getTopicId(),req.isTop());
        return "success";
    }

    @PostMapping(value = "/admin/setConceal")
    @ApiOperation("隐藏/显示话题")
    @ResponseBody
    public String setConceal(@RequestBody TopicConcealReq req) throws MissingServletRequestParameterException {
        topicManager.setTopicConceal(req.getTopicId(),req.isConceal());
        return "success";
    }

    @PostMapping(value = "/admin/modifyViewNum")
    @ApiOperation("话题浏览数调整")
    @ResponseBody
    public String modifyViewNum(@RequestBody TopicModifyMountReq req) throws MissingServletRequestParameterException {
        topicManager.modifyViewAmount(req.getTopicId(),req.getModifyMount());
        return "success";
    }

    @PostMapping(value = "/admin/dataAnalys")
    @ApiOperation("话题数据分析")
    @ResponseBody
    public TopicDataAnalysDTO dataAnalys(@RequestParam String topicId) throws MissingServletRequestParameterException {
        TopicDataAnalysDTO result =topicManager.queryDataAnalysis(topicId);
        return result;
    }



}
